@@ -1,10 +1,9 @@
 use std::{
     borrow::Cow,
-    collections::HashMap,
+    collections::{hash_map::Entry, HashMap},
     fmt::{self, Display, Formatter},
     ops::{BitXor, Range},
     str,
-    sync::RwLock,
 };
 
 use hmac::{digest::Digest, Hmac, Mac, NewMac};
@@ -12,6 +11,7 @@ use lazy_static::lazy_static;
 use md5::Md5;
 use sha1::Sha1;
 use sha2::Sha256;
+use tokio::sync::RwLock;
 
 use crate::{
     bson::{doc, Bson, Document},
@@ -154,7 +154,7 @@ impl ScramVersion {
 
         Ok(FirstRound {
             client_first,
-            server_first: server_first.raw_response,
+            server_first: server_first.auth_response_body("SCRAM")?,
         })
     }
 
@@ -190,7 +190,7 @@ impl ScramVersion {
             mechanism: self.clone(),
         };
         let (should_update_cache, salted_password) =
-            match CREDENTIAL_CACHE.read().unwrap().get(&cache_entry_key) {
+            match CREDENTIAL_CACHE.read().await.get(&cache_entry_key) {
                 Some(pwd) => (false, pwd.clone()),
                 None => (
                     true,
@@ -215,7 +215,7 @@ impl ScramVersion {
         let command = client_final.to_command();
 
         let server_final_response = conn.send_command(command, None).await?;
-        let server_final = ServerFinal::parse(server_final_response.raw_response)?;
+        let server_final = ServerFinal::parse(server_final_response.auth_response_body("SCRAM")?)?;
         server_final.validate(salted_password.as_slice(), &client_final, self)?;
 
         if !server_final.done {
@@ -231,9 +231,10 @@ impl ScramVersion {
             let command = noop.into_command();
 
             let server_noop_response = conn.send_command(command, None).await?;
+            let server_noop_response_document: Document =
+                server_noop_response.auth_response_body("SCRAM")?;
 
-            if server_noop_response
-                .raw_response
+            if server_noop_response_document
                 .get("conversationId")
                 .map(|id| id == server_final.conversation_id())
                 != Some(true)
@@ -244,8 +245,7 @@ impl ScramVersion {
                 ));
             };
 
-            if !server_noop_response
-                .raw_response
+            if !server_noop_response_document
                 .get_bool("done")
                 .unwrap_or(false)
             {
@@ -257,10 +257,9 @@ impl ScramVersion {
         }
 
         if should_update_cache {
-            if let Ok(ref mut cache) = CREDENTIAL_CACHE.write() {
-                if cache.get(&cache_entry_key).is_none() {
-                    cache.insert(cache_entry_key, salted_password);
-                }
+            let mut cache = CREDENTIAL_CACHE.write().await;
+            if let Entry::Vacant(entry) = cache.entry(cache_entry_key) {
+                entry.insert(salted_password);
             }
         }
 
