@@ -1,12 +1,20 @@
 use bson::doc;
 use lazy_static::lazy_static;
+use serde::de::DeserializeOwned;
 
 use std::{collections::HashSet, sync::Arc, time::Instant};
 
 use super::{session::TransactionState, Client, ClientSession};
 use crate::{
     bson::Document,
-    cmap::{Connection, RawCommand, RawCommandResponse},
+    cmap::{
+        conn::PinnedConnectionHandle,
+        Connection,
+        ConnectionPool,
+        RawCommand,
+        RawCommandResponse,
+    },
+    cursor::{session::SessionCursor, Cursor, CursorSpecification},
     error::{
         Error,
         ErrorKind,
@@ -70,6 +78,16 @@ impl Client {
         op: T,
         session: impl Into<Option<&mut ClientSession>>,
     ) -> Result<T::O> {
+        self.execute_operation_with_details(op, session)
+            .await
+            .map(|details| details.output.operation_output)
+    }
+
+    async fn execute_operation_with_details<T: Operation>(
+        &self,
+        op: T,
+        session: impl Into<Option<&mut ClientSession>>,
+    ) -> Result<ExecutionDetails<T>> {
         Box::pin(async {
             // TODO RUST-9: allow unacknowledged write concerns
             if !op.is_acknowledged() {
@@ -78,7 +96,8 @@ impl Client {
                 }
                 .into());
             }
-            match session.into() {
+            let mut implicit_session = None;
+            let session = match session.into() {
                 Some(session) => {
                     if !Arc::ptr_eq(&self.inner, &session.client().inner) {
                         return Err(ErrorKind::InvalidArgument {
@@ -99,41 +118,94 @@ impl Client {
                             .into());
                         }
                     }
-                    self.execute_operation_with_retry(op, Some(session)).await
+                    Some(session)
                 }
                 None => {
-                    let mut implicit_session = self.start_implicit_session(&op).await?;
-                    self.execute_operation_with_retry(op, implicit_session.as_mut())
-                        .await
+                    implicit_session = self.start_implicit_session(&op).await?;
+                    implicit_session.as_mut()
                 }
-            }
+            };
+            let output = self.execute_operation_with_retry(op, session).await?;
+            Ok(ExecutionDetails {
+                output,
+                implicit_session,
+            })
         })
         .await
     }
 
-    /// Execute the given operation, returning the implicit session created for it if one was.
+    /// Execute the given operation, returning the cursor created by the operation.
     ///
     /// Server selection be will performed using the criteria specified on the operation, if any.
-    pub(crate) async fn execute_cursor_operation<T: Operation>(
-        &self,
-        op: T,
-    ) -> Result<(T::O, Option<ClientSession>)> {
+    pub(crate) async fn execute_cursor_operation<Op, T>(&self, op: Op) -> Result<Cursor<T>>
+    where
+        Op: Operation<O = CursorSpecification<T>>,
+        T: DeserializeOwned + Unpin + Send + Sync,
+    {
         Box::pin(async {
-            let mut implicit_session = self.start_implicit_session(&op).await?;
-            self.execute_operation_with_retry(op, implicit_session.as_mut())
-                .await
-                .map(|result| (result, implicit_session))
+            let mut details = self.execute_operation_with_details(op, None).await?;
+            let pinned = self.pin_connection_for_cursor(&mut details.output)?;
+            Ok(Cursor::new(
+                self.clone(),
+                details.output.operation_output,
+                details.implicit_session,
+                pinned,
+            ))
         })
         .await
     }
 
+    pub(crate) async fn execute_session_cursor_operation<Op, T>(
+        &self,
+        op: Op,
+        session: &mut ClientSession,
+    ) -> Result<SessionCursor<T>>
+    where
+        Op: Operation<O = CursorSpecification<T>>,
+        T: DeserializeOwned + Unpin + Send + Sync,
+    {
+        let mut details = self
+            .execute_operation_with_details(op, &mut *session)
+            .await?;
+
+        let pinned = if let Some(handle) = session.transaction.pinned_connection() {
+            // Cursor operations on a transaction share the same pinned connection.
+            Some(handle.replicate())
+        } else {
+            self.pin_connection_for_cursor(&mut details.output)?
+        };
+        Ok(SessionCursor::new(
+            self.clone(),
+            details.output.operation_output,
+            pinned,
+        ))
+    }
+
+    fn is_load_balanced(&self) -> bool {
+        self.inner.options.load_balanced.unwrap_or(false)
+    }
+
+    fn pin_connection_for_cursor<Op, T>(
+        &self,
+        details: &mut ExecutionOutput<Op>,
+    ) -> Result<Option<PinnedConnectionHandle>>
+    where
+        Op: Operation<O = CursorSpecification<T>>,
+    {
+        if self.is_load_balanced() && details.operation_output.info.id != 0 {
+            Ok(Some(details.connection.pin()?))
+        } else {
+            Ok(None)
+        }
+    }
+
     /// Selects a server and executes the given operation on it, optionally using a provided
     /// session. Retries the operation upon failure if retryability is supported.
     async fn execute_operation_with_retry<T: Operation>(
         &self,
         mut op: T,
         mut session: Option<&mut ClientSession>,
-    ) -> Result<T::O> {
+    ) -> Result<ExecutionOutput<T>> {
         // If the current transaction has been committed/aborted and it is not being
         // re-committed/re-aborted, reset the transaction's state to TransactionState::None.
         if let Some(ref mut session) = session {
@@ -150,7 +222,7 @@ impl Client {
 
         let selection_criteria = session
             .as_ref()
-            .and_then(|s| s.transaction.pinned_mongos.as_ref())
+            .and_then(|s| s.transaction.pinned_mongos())
             .or_else(|| op.selection_criteria());
 
         let server = match self.select_server(selection_criteria).await {
@@ -161,8 +233,8 @@ impl Client {
             }
         };
 
-        let mut conn = match server.pool.check_out().await {
-            Ok(conn) => conn,
+        let mut conn = match get_connection(&session, &op, &server.pool).await {
+            Ok(c) => c,
             Err(mut err) => {
                 err.add_labels_and_update_pin(None, &mut session, None)?;
 
@@ -200,7 +272,10 @@ impl Client {
             )
             .await
         {
-            Ok(result) => Ok(result),
+            Ok(operation_output) => Ok(ExecutionOutput {
+                operation_output,
+                connection: conn,
+            }),
             Err(mut err) => {
                 // Retryable writes are only supported by storage engines with document-level
                 // locking, so users need to disable retryable writes if using mmapv1.
@@ -246,7 +321,7 @@ impl Client {
         session: &mut Option<&mut ClientSession>,
         txn_number: Option<i64>,
         first_error: Error,
-    ) -> Result<T::O> {
+    ) -> Result<ExecutionOutput<T>> {
         op.update_for_retry();
 
         let server = match self.select_server(op.selection_criteria()).await {
@@ -256,7 +331,7 @@ impl Client {
             }
         };
 
-        let mut conn = match server.pool.check_out().await {
+        let mut conn = match get_connection(session, op, &server.pool).await {
             Ok(c) => c,
             Err(_) => return Err(first_error),
         };
@@ -270,7 +345,10 @@ impl Client {
             .execute_operation_on_connection(op, &mut conn, session, txn_number, &retryability)
             .await
         {
-            Ok(result) => Ok(result),
+            Ok(operation_output) => Ok(ExecutionOutput {
+                operation_output,
+                connection: conn,
+            }),
             Err(err) => {
                 self.inner
                     .topology
@@ -339,12 +417,34 @@ impl Client {
                     }
                     cmd.set_snapshot_read_concern(session);
                 }
+                // If this is a causally consistent session, set `readConcern.afterClusterTime`.
+                // Causal consistency defaults to true, unless snapshot is true.
+                else if session
+                    .options()
+                    .and_then(|opts| opts.causal_consistency)
+                    .unwrap_or(true)
+                    && matches!(
+                        session.transaction.state,
+                        TransactionState::None | TransactionState::Starting
+                    )
+                    && op.supports_read_concern(stream_description)
+                {
+                    cmd.set_after_cluster_time(session);
+                }
+
                 match session.transaction.state {
                     TransactionState::Starting => {
                         cmd.set_start_transaction();
                         cmd.set_autocommit();
-                        cmd.set_txn_read_concern(*session);
-                        if is_sharded {
+
+                        if let Some(ref options) = session.transaction.options {
+                            if let Some(ref read_concern) = options.read_concern {
+                                cmd.set_read_concern_level(read_concern.level.clone());
+                            }
+                        }
+                        if self.is_load_balanced() {
+                            session.pin_connection(connection.pin()?);
+                        } else if is_sharded {
                             session.pin_mongos(connection.address().clone());
                         }
                         session.transaction.state = TransactionState::InProgress;
@@ -389,6 +489,7 @@ impl Client {
         }
 
         let connection_info = connection.info();
+        let service_id = connection.service_id();
         let request_id = crate::cmap::conn::next_request_id();
 
         if let Some(ref server_api) = self.inner.options.server_api {
@@ -420,6 +521,7 @@ impl Client {
                 command_name: raw_cmd.name.clone(),
                 request_id,
                 connection: connection_info.clone(),
+                service_id,
             };
 
             handler.handle_command_started_event(command_started_event);
@@ -430,6 +532,9 @@ impl Client {
             Ok(response) => {
                 match T::Response::deserialize_response(&response) {
                     Ok(r) => {
+                        if let (Some(session), Some(ts)) = (session.as_mut(), r.operation_time()) {
+                            session.advance_operation_time(ts);
+                        }
                         self.update_cluster_time(&r, session).await;
                         if r.is_success() {
                             // Retrieve recovery token from successful response.
@@ -460,6 +565,11 @@ impl Client {
                         // a generic command response without the operation's body.
                         match response.body::<CommandResponse<Option<CommandErrorBody>>>() {
                             Ok(error_response) => {
+                                if let (Some(session), Some(ts)) =
+                                    (session.as_mut(), error_response.operation_time())
+                                {
+                                    session.advance_operation_time(ts);
+                                }
                                 self.update_cluster_time(&error_response, session).await;
                                 match error_response.body {
                                     // if the response was ok: 0, return the command error.
@@ -511,6 +621,7 @@ impl Client {
                         failure: err.clone(),
                         request_id,
                         connection: connection_info,
+                        service_id,
                     };
 
                     handler.handle_command_failed_event(command_failed_event);
@@ -542,6 +653,7 @@ impl Client {
                         command_name: cmd_name.clone(),
                         request_id,
                         connection: connection_info,
+                        service_id,
                     };
                     handler.handle_command_succeeded_event(command_succeeded_event);
                 });
@@ -683,6 +795,25 @@ impl Client {
     }
 }
 
+async fn get_connection<T: Operation>(
+    session: &Option<&mut ClientSession>,
+    op: &T,
+    pool: &ConnectionPool,
+) -> Result<Connection> {
+    let session_pinned = session
+        .as_ref()
+        .and_then(|s| s.transaction.pinned_connection());
+    match (session_pinned, op.pinned_connection()) {
+        (Some(c), None) | (None, Some(c)) => c.take_connection().await,
+        (Some(session_handle), Some(op_handle)) => {
+            // An operation executing in a transaction should be sharing the same pinned connection.
+            debug_assert_eq!(session_handle.id(), op_handle.id());
+            session_handle.take_connection().await
+        }
+        (None, None) => pool.check_out().await,
+    }
+}
+
 impl Error {
     /// Adds the necessary labels to this Error, and unpins the session if needed.
     ///
@@ -749,7 +880,7 @@ impl Error {
             if self.contains_label(TRANSIENT_TRANSACTION_ERROR)
                 || self.contains_label(UNKNOWN_TRANSACTION_COMMIT_RESULT)
             {
-                session.unpin_mongos();
+                session.unpin();
             }
         }
 
@@ -761,3 +892,13 @@ struct CommandResult<T> {
     raw: RawCommandResponse,
     deserialized: T,
 }
+
+struct ExecutionDetails<T: Operation> {
+    output: ExecutionOutput<T>,
+    implicit_session: Option<ClientSession>,
+}
+
+struct ExecutionOutput<T: Operation> {
+    operation_output: T::O,
+    connection: Connection,
+}
