@@ -69,6 +69,14 @@ const DEFAULT_SERVER_SELECTION_TIMEOUT: Duration = Duration::from_secs(30);
 /// # Ok(())
 /// # }
 /// ```
+/// ## Notes on performance
+/// Spawning many asynchronous tasks that use the driver concurrently like this is often the best
+/// way to achieve maximum performance, as the driver is designed to work well in such situations.
+///
+/// Additionally, using a custom Rust type that implements `Serialize` and `Deserialize` as the
+/// generic parameter of [`Collection`](../struct.Collection.html) instead of [`bson::Document`] can
+/// reduce the amount of time the driver and your application spends serializing and deserializing
+/// BSON, which can also lead to increased performance.
 ///
 /// ## TCP Keepalive
 /// TCP keepalive is enabled by default with ``tcp_keepalive_time`` set to 120 seconds. The
@@ -160,6 +168,18 @@ impl Client {
         Database::new(self.clone(), name, Some(options))
     }
 
+    /// Gets a handle to the default database specified in the `ClientOptions` or MongoDB connection
+    /// string used to construct this `Client`.
+    ///
+    /// If no default database was specified, `None` will be returned.
+    pub fn default_database(&self) -> Option<Database> {
+        self.inner
+            .options
+            .default_database
+            .as_ref()
+            .map(|db_name| self.database(db_name))
+    }
+
     async fn list_databases_common(
         &self,
         filter: impl Into<Option<Document>>,
@@ -222,6 +242,9 @@ impl Client {
 
     /// Starts a new `ClientSession`.
     pub async fn start_session(&self, options: Option<SessionOptions>) -> Result<ClientSession> {
+        if let Some(ref options) = options {
+            options.validate()?;
+        }
         match self.get_session_support_status().await? {
             SessionSupportStatus::Supported {
                 logical_session_timeout,
@@ -344,6 +367,11 @@ impl Client {
         }
     }
 
+    #[cfg(test)]
+    pub(crate) async fn topology_description(&self) -> crate::sdam::TopologyDescription {
+        self.inner.topology.description().await
+    }
+
     #[cfg(all(test, not(feature = "sync")))]
     pub(crate) async fn get_hosts(&self) -> Vec<String> {
         let servers = self.inner.topology.servers().await;
@@ -352,4 +380,9 @@ impl Client {
             .map(|stream_address| format!("{}", stream_address))
             .collect()
     }
+
+    #[cfg(test)]
+    pub(crate) async fn sync_workers(&self) {
+        self.inner.topology.sync_workers().await;
+    }
 }
