@@ -26,6 +26,7 @@ use rustls::{
     ServerCertVerifier,
     TLSError,
 };
+use rustls_pemfile::{read_one, Item};
 use serde::{
     de::{Error, Unexpected},
     Deserialize,
@@ -37,10 +38,13 @@ use strsim::jaro_winkler;
 use typed_builder::TypedBuilder;
 use webpki_roots::TLS_SERVER_ROOTS;
 
+#[cfg(test)]
+use crate::srv::LookupHosts;
 use crate::{
     bson::{doc, Bson, Document},
     bson_util,
     client::auth::{AuthMechanism, Credential},
+    compression::Compressor,
     concern::{Acknowledgment, ReadConcern, WriteConcern},
     error::{ErrorKind, Result},
     event::{cmap::CmapEventHandler, command::CommandEventHandler, sdam::SdamEventHandler},
@@ -168,14 +172,31 @@ impl Hash for ServerAddress {
     }
 }
 
+impl FromStr for ServerAddress {
+    type Err = crate::error::Error;
+    fn from_str(address: &str) -> Result<Self> {
+        ServerAddress::parse(address)
+    }
+}
+
 impl ServerAddress {
-    /// Parses an address string into a `StreamAddress`.
+    /// Parses an address string into a `ServerAddress`.
     pub fn parse(address: impl AsRef<str>) -> Result<Self> {
         let address = address.as_ref();
         let mut parts = address.split(':');
-
         let hostname = match parts.next() {
-            Some(part) => part,
+            Some(part) => {
+                if part.is_empty() {
+                    return Err(ErrorKind::InvalidArgument {
+                        message: format!(
+                            "invalid server address: \"{}\"; hostname cannot be empty",
+                            address
+                        ),
+                    }
+                    .into());
+                }
+                part
+            }
             None => {
                 return Err(ErrorKind::InvalidArgument {
                     message: format!("invalid server address: \"{}\"", address),
@@ -193,6 +214,15 @@ impl ServerAddress {
                     ),
                 })?;
 
+                if port == 0 {
+                    return Err(ErrorKind::InvalidArgument {
+                        message: format!(
+                            "invalid server address: \"{}\"; port must be non-zero",
+                            address
+                        ),
+                    }
+                    .into());
+                }
                 if parts.next().is_some() {
                     return Err(ErrorKind::InvalidArgument {
                         message: format!(
@@ -209,7 +239,7 @@ impl ServerAddress {
         };
 
         Ok(ServerAddress::Tcp {
-            host: hostname.to_string(),
+            host: hostname.to_lowercase(),
             port,
         })
     }
@@ -342,8 +372,13 @@ pub struct ClientOptions {
     #[builder(default)]
     pub app_name: Option<String>,
 
-    #[builder(default, setter(skip))]
-    pub(crate) compressors: Option<Vec<String>>,
+    /// The compressors that the Client is willing to use in the order they are specified
+    /// in the configuration.  The Client sends this list of compressors to the server.
+    /// The server responds with the intersection of its supported list of compressors.
+    /// The order of compressors indicates preference of compressors.
+    #[builder(default)]
+    #[serde(skip)]
+    pub compressors: Option<Vec<Compressor>>,
 
     /// The handler that should process all Connection Monitoring and Pooling events. See the
     /// CmapEventHandler type documentation for more details.
@@ -391,6 +426,11 @@ pub struct ClientOptions {
     #[builder(default)]
     pub heartbeat_freq: Option<Duration>,
 
+    /// Whether or not the client is connecting to a MongoDB cluster through a load balancer.
+    #[builder(default, setter(skip))]
+    #[serde(rename = "loadbalanced")]
+    pub load_balanced: Option<bool>,
+
     /// When running a read operation with a ReadPreference that allows selecting secondaries,
     /// `local_threshold` is used to determine how much longer the average round trip time between
     /// the driver and server is allowed compared to the least round trip time of all the suitable
@@ -483,6 +523,12 @@ pub struct ClientOptions {
     #[builder(default)]
     pub server_selection_timeout: Option<Duration>,
 
+    /// Default database for this client.
+    ///
+    /// By default, no default database is specified.
+    #[builder(default)]
+    pub default_database: Option<String>,
+
     #[builder(default, setter(skip))]
     #[derivative(Debug = "ignore")]
     pub(crate) socket_timeout: Option<Duration>,
@@ -498,9 +544,6 @@ pub struct ClientOptions {
     #[builder(default)]
     pub write_concern: Option<WriteConcern>,
 
-    #[builder(default, setter(skip))]
-    pub(crate) zlib_compression: Option<i32>,
-
     /// Information from the SRV URI that generated these client options, if applicable.
     #[builder(default, setter(skip))]
     #[serde(skip)]
@@ -521,20 +564,28 @@ pub struct ClientOptions {
     #[derivative(Debug = "ignore")]
     pub(crate) resolver_config: Option<ResolverConfig>,
 
-    /// Used by tests to override MIN_HEARTBEAT_FREQUENCY.
-    #[builder(default)]
+    /// Control test behavior of the client.
     #[cfg(test)]
-    pub(crate) heartbeat_freq_test: Option<Duration>,
-
-    /// Allow use of the `load_balanced` option.
-    // TODO RUST-653 Remove this when load balancer work is ready for release.
     #[builder(default, setter(skip))]
     #[serde(skip)]
-    pub(crate) allow_load_balanced: bool,
+    #[derivative(PartialEq = "ignore")]
+    pub(crate) test_options: Option<TestOptions>,
+}
 
-    /// Whether or not the client is connecting to a MongoDB cluster through a load balancer.
-    #[builder(default, setter(skip))]
-    pub(crate) load_balanced: Option<bool>,
+#[cfg(test)]
+#[derive(Debug, Clone, Default)]
+pub(crate) struct TestOptions {
+    /// Override MIN_HEARTBEAT_FREQUENCY.
+    pub(crate) heartbeat_freq: Option<Duration>,
+
+    /// Disable server and SRV-polling monitor threads.
+    pub(crate) disable_monitoring_threads: bool,
+
+    /// Mock response for `SrvPollingMonitor::lookup_hosts`.
+    pub(crate) mock_lookup_hosts: Option<Result<LookupHosts>>,
+
+    /// Mock the `serviceId` response for a load-balanced hello.
+    pub(crate) mock_service_id: bool,
 }
 
 fn default_hosts() -> Vec<ServerAddress> {
@@ -556,7 +607,6 @@ impl Serialize for ClientOptions {
         #[derive(Serialize)]
         struct ClientOptionsHelper<'a> {
             appname: &'a Option<String>,
-            compressors: &'a Option<Vec<String>>,
 
             #[serde(serialize_with = "bson_util::serialize_duration_option_as_int_millis")]
             connecttimeoutms: &'a Option<Duration>,
@@ -607,11 +657,12 @@ impl Serialize for ClientOptions {
             writeconcern: &'a Option<WriteConcern>,
 
             zlibcompressionlevel: &'a Option<i32>,
+
+            loadbalanced: &'a Option<bool>,
         }
 
         let client_options = ClientOptionsHelper {
             appname: &self.app_name,
-            compressors: &self.compressors,
             connecttimeoutms: &self.connect_timeout,
             credential: &self.credential,
             directconnection: &self.direct_connection,
@@ -629,7 +680,8 @@ impl Serialize for ClientOptions {
             sockettimeoutms: &self.socket_timeout,
             tls: &self.tls,
             writeconcern: &self.write_concern,
-            zlibcompressionlevel: &self.zlib_compression,
+            loadbalanced: &self.load_balanced,
+            zlibcompressionlevel: &None,
         };
 
         client_options.serialize(serializer)
@@ -653,7 +705,7 @@ struct ClientOptionsParser {
     pub min_pool_size: Option<u32>,
     pub max_idle_time: Option<Duration>,
     pub wait_queue_timeout: Option<Duration>,
-    pub compressors: Option<Vec<String>>,
+    pub compressors: Option<Vec<Compressor>>,
     pub connect_timeout: Option<Duration>,
     pub retry_reads: Option<bool>,
     pub retry_writes: Option<bool>,
@@ -661,6 +713,7 @@ struct ClientOptionsParser {
     pub zlib_compression: Option<i32>,
     pub direct_connection: Option<bool>,
     pub credential: Option<Credential>,
+    pub default_database: Option<String>,
     max_staleness: Option<Duration>,
     tls_insecure: Option<bool>,
     auth_mechanism: Option<AuthMechanism>,
@@ -794,22 +847,32 @@ impl TlsOptions {
             };
 
             file.seek(SeekFrom::Start(0))?;
-            let key = match pemfile::rsa_private_keys(&mut file) {
-                Ok(key) => key,
-                Err(()) => {
-                    return Err(ErrorKind::InvalidTlsConfig {
-                        message: format!(
-                            "Unable to parse PEM-encoded RSA key from {}",
-                            path.display()
-                        ),
+            let key = loop {
+                match read_one(&mut file) {
+                    Ok(Some(Item::PKCS8Key(bytes))) | Ok(Some(Item::RSAKey(bytes))) => {
+                        break rustls::PrivateKey(bytes)
+                    }
+                    Ok(Some(_)) => continue,
+                    Ok(None) => {
+                        return Err(ErrorKind::InvalidTlsConfig {
+                            message: format!("No PEM-encoded keys in {}", path.display()),
+                        }
+                        .into())
+                    }
+                    Err(_) => {
+                        return Err(ErrorKind::InvalidTlsConfig {
+                            message: format!(
+                                "Unable to parse PEM-encoded item from {}",
+                                path.display()
+                            ),
+                        }
+                        .into())
                     }
-                    .into())
                 }
             };
 
-            // TODO: Get rid of unwrap.
             config
-                .set_single_client_cert(certs, key.into_iter().next().unwrap())
+                .set_single_client_cert(certs, key)
                 .map_err(|e| ErrorKind::InvalidTlsConfig {
                     message: e.to_string(),
                 })?;
@@ -889,8 +952,8 @@ impl From<ClientOptionsParser> for ClientOptions {
             retry_reads: parser.retry_reads,
             retry_writes: parser.retry_writes,
             socket_timeout: parser.socket_timeout,
-            zlib_compression: parser.zlib_compression,
             direct_connection: parser.direct_connection,
+            default_database: parser.default_database,
             driver_info: None,
             credential: parser.credential,
             cmap_event_handler: None,
@@ -899,11 +962,10 @@ impl From<ClientOptionsParser> for ClientOptions {
             original_uri: Some(parser.original_uri),
             resolver_config: None,
             server_api: None,
-            #[cfg(test)]
-            heartbeat_freq_test: None,
-            allow_load_balanced: false,
             load_balanced: parser.load_balanced,
             sdam_event_handler: None,
+            #[cfg(test)]
+            test_options: None,
         }
     }
 }
@@ -930,6 +992,9 @@ impl ClientOptions {
     ///
     /// The format of a MongoDB connection string is described [here](https://docs.mongodb.com/manual/reference/connection-string/#connection-string-formats).
     ///
+    /// Note that [default_database](ClientOptions::default_database) will be set from
+    /// `/defaultauthdb` in connection string.
+    ///
     /// The following options are supported in the options query string:
     ///
     ///   * `appName`: maps to the `app_name` field
@@ -937,7 +1002,7 @@ impl ClientOptions {
     ///   * `authSource`: maps to the `source` field of the `credential` field
     ///   * `authMechanismProperties`: maps to the `mechanism_properties` field of the `credential`
     ///     field
-    ///   * `compressors`: not yet implemented
+    ///   * `compressors`: maps to the `compressors` field
     ///   * `connectTimeoutMS`: maps to the `connect_timeout` field
     ///   * `direct`: maps to the `direct` field
     ///   * `heartbeatFrequencyMS`: maps to the `heartbeat_frequency` field
@@ -971,7 +1036,8 @@ impl ClientOptions {
     ///   * `w`: maps to the `w` field of the `write_concern` field
     ///   * `waitQueueTimeoutMS`: unsupported, does not map to any field
     ///   * `wTimeoutMS`: maps to the `w_timeout` field of the `write_concern` field
-    ///   * `zlibCompressionLevel`: not yet implemented
+    ///   * `zlibCompressionLevel`: maps to the `level` field of the `Compressor::Zlib` variant
+    ///     (which requires the `zlib-compression` feature flag) of the [`Compressor`] enum
     ///
     /// Note: if the `sync` feature is enabled, then this method will be replaced with [the sync
     /// version](#method.parse-1).
@@ -1110,13 +1176,6 @@ impl ClientOptions {
             write_concern.validate()?;
         }
 
-        if !self.allow_load_balanced && self.load_balanced.is_some() {
-            return Err(ErrorKind::InvalidArgument {
-                message: "loadBalanced is not supported".to_string(),
-            }
-            .into());
-        }
-
         if self.load_balanced.unwrap_or(false) {
             if self.hosts.len() > 1 {
                 return Err(ErrorKind::InvalidArgument {
@@ -1139,12 +1198,27 @@ impl ClientOptions {
             }
         }
 
+        if let Some(ref compressors) = self.compressors {
+            for compressor in compressors {
+                compressor.validate()?;
+            }
+        }
+
+        if let Some(0) = self.max_pool_size {
+            return Err(crate::error::Error::invalid_argument(
+                "cannot specify maxPoolSize=0",
+            ));
+        }
+
         Ok(())
     }
 
     /// Applies the options in other to these options if a value is not already present
     #[cfg(test)]
     pub(crate) fn merge(&mut self, other: ClientOptions) {
+        if self.hosts.is_empty() {
+            self.hosts = other.hosts;
+        }
         merge_options!(
             other,
             self,
@@ -1158,6 +1232,7 @@ impl ClientOptions {
                 direct_connection,
                 driver_info,
                 heartbeat_freq,
+                load_balanced,
                 local_threshold,
                 max_idle_time,
                 max_pool_size,
@@ -1170,14 +1245,19 @@ impl ClientOptions {
                 server_api,
                 server_selection_timeout,
                 socket_timeout,
+                test_options,
                 tls,
                 write_concern,
-                zlib_compression,
                 original_srv_info,
                 original_uri
             ]
         );
     }
+
+    #[cfg(test)]
+    pub(crate) fn test_options_mut(&mut self) -> &mut TestOptions {
+        self.test_options.get_or_insert_with(Default::default)
+    }
 }
 
 /// Splits a string into a section before a given index and a section exclusively after the index.
@@ -1324,52 +1404,7 @@ impl ClientOptionsParser {
             None => (None, None),
         };
 
-        let hosts: Result<Vec<_>> = hosts_section
-            .split(',')
-            .map(|host| {
-                let (hostname, port) = match host.find(':') {
-                    Some(index) => host.split_at(index),
-                    None => (host, ""),
-                };
-
-                if hostname.is_empty() {
-                    return Err(ErrorKind::InvalidArgument {
-                        message: "connection string contains no host".to_string(),
-                    }
-                    .into());
-                }
-                let port = if port.is_empty() {
-                    None
-                } else {
-                    let port_string_without_colon = &port[1..];
-                    let p: u16 = port_string_without_colon.parse().map_err(|_| {
-                        ErrorKind::InvalidArgument {
-                            message: format!(
-                                "invalid port specified in connection string: {}",
-                                port
-                            ),
-                        }
-                    })?;
-
-                    if p == 0 {
-                        return Err(ErrorKind::InvalidArgument {
-                            message: format!(
-                                "invalid port specified in connection string: {}",
-                                port
-                            ),
-                        }
-                        .into());
-                    }
-
-                    Some(p)
-                };
-
-                Ok(ServerAddress::Tcp {
-                    host: hostname.to_lowercase(),
-                    port,
-                })
-            })
-            .collect();
+        let hosts: Result<Vec<_>> = hosts_section.split(',').map(ServerAddress::parse).collect();
 
         let hosts = hosts?;
 
@@ -1464,7 +1499,7 @@ impl ClientOptionsParser {
                     credential.source = options
                         .auth_source
                         .clone()
-                        .or(db)
+                        .or_else(|| db.clone())
                         .or_else(|| Some("admin".into()));
                 } else if authentication_requested {
                     return Err(ErrorKind::InvalidArgument {
@@ -1477,6 +1512,9 @@ impl ClientOptionsParser {
             }
         };
 
+        // set default database.
+        options.default_database = db;
+
         if options.tls.is_none() && options.srv {
             options.tls = Some(Tls::Enabled(Default::default()));
         }
@@ -1563,6 +1601,16 @@ impl ClientOptionsParser {
             }
         }
 
+        // If zlib and zlib_compression_level are specified then write zlib_compression_level into
+        // zlib enum
+        if let (Some(compressors), Some(zlib_compression_level)) =
+            (self.compressors.as_mut(), self.zlib_compression)
+        {
+            for compressor in compressors {
+                compressor.write_zlib_level(zlib_compression_level)
+            }
+        }
+
         Ok(())
     }
 
@@ -1667,7 +1715,15 @@ impl ClientOptionsParser {
                 self.auth_mechanism_properties = Some(doc);
             }
             "compressors" => {
-                self.compressors = Some(value.split(',').map(String::from).collect());
+                let compressors = value
+                    .split(',')
+                    .filter_map(|x| Compressor::parse_str(x).ok())
+                    .collect::<Vec<Compressor>>();
+                self.compressors = if compressors.is_empty() {
+                    None
+                } else {
+                    Some(compressors)
+                }
             }
             k @ "connecttimeoutms" => {
                 self.connect_timeout = Some(Duration::from_millis(get_duration!(value, k)));
@@ -1998,6 +2054,23 @@ mod tests {
         }
     }
 
+    #[test]
+    fn test_parse_address_with_from_str() {
+        let x = "localhost:27017".parse::<ServerAddress>().unwrap();
+        let ServerAddress::Tcp { host, port } = x;
+        assert_eq!(host, "localhost");
+        assert_eq!(port, Some(27017));
+
+        // Port defaults to 27017 (so this doesn't fail)
+        let x = "localhost".parse::<ServerAddress>().unwrap();
+        let ServerAddress::Tcp { host, port } = x;
+        assert_eq!(host, "localhost");
+        assert_eq!(port, None);
+
+        let x = "localhost:not a number".parse::<ServerAddress>();
+        assert!(x.is_err());
+    }
+
     #[cfg_attr(feature = "tokio-runtime", tokio::test)]
     #[cfg_attr(feature = "async-std-runtime", async_std::test)]
     async fn fails_without_scheme() {
@@ -2338,12 +2411,32 @@ pub struct SessionOptions {
     /// associated with the operations within the transaction.
     pub default_transaction_options: Option<TransactionOptions>,
 
+    /// If true, all operations performed in the context of this session
+    /// will be [causally consistent](https://docs.mongodb.com/manual/core/causal-consistency-read-write-concerns/).
+    ///
+    /// Defaults to true if [`SessionOptions::snapshot`] is unspecified.
+    pub causal_consistency: Option<bool>,
+
     /// If true, all read operations performed using this client session will share the same
     /// snapshot.  Defaults to false.
-    // TODO RUST-18 enforce snapshot exclusivity with causalConsistency.
     pub snapshot: Option<bool>,
 }
 
+impl SessionOptions {
+    pub(crate) fn validate(&self) -> Result<()> {
+        if let (Some(causal_consistency), Some(snapshot)) = (self.causal_consistency, self.snapshot)
+        {
+            if causal_consistency && snapshot {
+                return Err(ErrorKind::InvalidArgument {
+                    message: "snapshot and causal consistency are mutually exclusive".to_string(),
+                }
+                .into());
+            }
+        }
+        Ok(())
+    }
+}
+
 /// Contains the options that can be used for a transaction.
 #[skip_serializing_none]
 #[derive(Debug, Default, Serialize, Deserialize, TypedBuilder, Clone)]
