@@ -5,6 +5,7 @@ use crate::{
     bson::{Bson, Document},
     client::options::{ClientOptions, ClientOptionsParser, ServerAddress},
     error::ErrorKind,
+    options::Compressor,
     test::run_spec_test,
 };
 #[derive(Debug, Deserialize)]
@@ -40,6 +41,20 @@ async fn run_test(test_file: TestFile) {
             || test_case.description.contains("serverSelectionTryOnce")
             || test_case.description.contains("Unix")
             || test_case.description.contains("relative path")
+            // Compression is implemented but will only pass the tests if all
+            // the appropriate feature flags are set.  That is because
+            // valid compressors are only parsed correctly if the corresponding feature flag is set.
+            // (otherwise they are treated as invalid, and hence ignored)
+            || (test_case.description.contains("compress") &&
+                    !cfg!(
+                        all(features = "zlib-compression",
+                            features = "zstd-compression",
+                            features = "snappy-compression"
+                        )
+                    )
+                )
+            // The Rust driver disallows `maxPoolSize=0`.
+            || test_case.description.contains("maxPoolSize=0 does not error")
         {
             continue;
         }
@@ -100,11 +115,39 @@ async fn run_test(test_file: TestFile) {
                         }
                     }
 
+                    // The default types parsed from the test file don't match those serialized
+                    // from the `ClientOptions` struct.
+                    if let Ok(min) = json_options.get_i32("minpoolsize") {
+                        json_options.insert("minpoolsize", Bson::Int64(min.into()));
+                    }
+                    if let Ok(max) = json_options.get_i32("maxpoolsize") {
+                        json_options.insert("maxpoolsize", Bson::Int64(max.into()));
+                    }
+
                     options_doc = options_doc
                         .into_iter()
                         .filter(|(ref key, _)| json_options.contains_key(key))
                         .collect();
 
+                    // This is required because compressor is not serialize, but the spec tests
+                    // still expect to see serialized compressors.
+                    // This hardcodes the compressors into the options.
+                    if let Some(compressors) = options.compressors {
+                        options_doc.insert(
+                            "compressors",
+                            compressors
+                                .iter()
+                                .map(Compressor::name)
+                                .collect::<Vec<&str>>(),
+                        );
+                        #[cfg(feature = "zlib-compression")]
+                        for compressor in compressors {
+                            if let Compressor::Zlib { level: Some(level) } = compressor {
+                                options_doc.insert("zlibcompressionlevel", level);
+                            }
+                        }
+                    }
+
                     assert_eq!(options_doc, json_options, "{}", test_case.description)
                 }
                 // auth
@@ -190,6 +233,44 @@ async fn parse_unknown_options() {
     parse_uri("maxstalenessms", Some("maxstalenessseconds")).await;
 }
 
+#[cfg_attr(feature = "tokio-runtime", tokio::test)]
+#[cfg_attr(feature = "async-std-runtime", async_std::test)]
+async fn parse_with_default_database() {
+    let uri = "mongodb://localhost/abc";
+
+    assert_eq!(
+        ClientOptions::parse(uri).await.unwrap(),
+        ClientOptions {
+            hosts: vec![ServerAddress::Tcp {
+                host: "localhost".to_string(),
+                port: None
+            }],
+            original_uri: Some(uri.into()),
+            default_database: Some("abc".to_string()),
+            ..Default::default()
+        }
+    );
+}
+
+#[cfg_attr(feature = "tokio-runtime", tokio::test)]
+#[cfg_attr(feature = "async-std-runtime", async_std::test)]
+async fn parse_with_no_default_database() {
+    let uri = "mongodb://localhost/";
+
+    assert_eq!(
+        ClientOptions::parse(uri).await.unwrap(),
+        ClientOptions {
+            hosts: vec![ServerAddress::Tcp {
+                host: "localhost".to_string(),
+                port: None
+            }],
+            original_uri: Some(uri.into()),
+            default_database: None,
+            ..Default::default()
+        }
+    );
+}
+
 #[cfg_attr(feature = "tokio-runtime", tokio::test)]
 #[cfg_attr(feature = "async-std-runtime", async_std::test)]
 async fn options_debug_omits_uri() {
