@@ -10,19 +10,16 @@ use crate::bson::{Document, Timestamp};
 #[derive(Debug, Deserialize, Clone, Serialize, Derivative)]
 #[derivative(PartialEq, Eq)]
 #[serde(rename_all = "camelCase")]
-pub(crate) struct ClusterTime {
-    cluster_time: Timestamp,
+pub struct ClusterTime {
+    pub(crate) cluster_time: Timestamp,
 
     #[derivative(PartialEq = "ignore")]
-    signature: Document,
+    pub(crate) signature: Document,
 }
 
 impl std::cmp::Ord for ClusterTime {
-    // TODO: RUST-390 use Timestamp's Ord impl.
     fn cmp(&self, other: &ClusterTime) -> std::cmp::Ordering {
-        let lhs = (self.cluster_time.time, self.cluster_time.increment);
-        let rhs = (other.cluster_time.time, other.cluster_time.increment);
-        lhs.cmp(&rhs)
+        self.cluster_time.cmp(&other.cluster_time)
     }
 }
 
