@@ -12,6 +12,8 @@ mod worker;
 use std::sync::Arc;
 
 use derivative::Derivative;
+#[cfg(test)]
+use tokio::sync::oneshot;
 
 pub use self::conn::ConnectionInfo;
 pub(crate) use self::{
@@ -96,8 +98,8 @@ impl ConnectionPool {
     pub(crate) fn new_mocked(address: ServerAddress) -> Self {
         let (manager, _) = manager::channel();
         let handle = PoolWorkerHandle::new_mocked();
-        let (connection_requester, _) = connection_requester::channel(Default::default(), handle);
-        let (_, generation_subscriber) = status::channel();
+        let (connection_requester, _) = connection_requester::channel(handle);
+        let (_, generation_subscriber) = status::channel(PoolGeneration::normal());
 
         Self {
             address,
@@ -132,7 +134,7 @@ impl ConnectionPool {
         let response = self.connection_requester.request().await;
 
         let conn = match response {
-            ConnectionRequestResult::Pooled(c) => Ok(c),
+            ConnectionRequestResult::Pooled(c) => Ok(*c),
             ConnectionRequestResult::Establishing(task) => task.await,
             ConnectionRequestResult::PoolCleared(e) => {
                 Err(Error::pool_cleared_error(&self.address, &e))
@@ -172,4 +174,9 @@ impl ConnectionPool {
     pub(crate) fn generation(&self) -> PoolGeneration {
         self.generation_subscriber.generation()
     }
+
+    #[cfg(test)]
+    pub(crate) fn sync_worker(&self) -> oneshot::Receiver<()> {
+        self.manager.sync_worker()
+    }
 }
