@@ -171,10 +171,8 @@ impl ConnectionPoolWorker {
             .map(|pool_options| ConnectionOptions::from(pool_options.clone()));
 
         let (handle, handle_listener) = handle_channel();
-        let (connection_requester, request_receiver) =
-            connection_requester::channel(address.clone(), handle);
+        let (connection_requester, request_receiver) = connection_requester::channel(handle);
         let (manager, management_receiver) = manager::channel();
-        let (generation_publisher, generation_subscriber) = status::channel();
 
         let is_load_balanced = options
             .as_ref()
@@ -185,6 +183,7 @@ impl ConnectionPoolWorker {
         } else {
             PoolGeneration::normal()
         };
+        let (generation_publisher, generation_subscriber) = status::channel(generation.clone());
 
         #[cfg(test)]
         let mut state = if options
@@ -292,27 +291,32 @@ impl ConnectionPoolWorker {
                         ));
                     }
                 },
-                PoolTask::HandleManagementRequest(PoolManagementRequest::CheckIn(connection)) => {
-                    self.check_in(connection);
-                }
-                PoolTask::HandleManagementRequest(PoolManagementRequest::Clear {
-                    completion_handler: _,
-                    cause,
-                    service_id,
-                }) => {
-                    self.clear(cause, service_id);
-                }
-                PoolTask::HandleManagementRequest(PoolManagementRequest::MarkAsReady {
-                    completion_handler: _handler,
-                }) => {
-                    self.mark_as_ready();
-                }
-                PoolTask::HandleManagementRequest(
-                    PoolManagementRequest::HandleConnectionSucceeded(conn),
-                ) => self.handle_connection_succeeded(conn),
-                PoolTask::HandleManagementRequest(
-                    PoolManagementRequest::HandleConnectionFailed,
-                ) => self.handle_connection_failed(),
+                PoolTask::HandleManagementRequest(request) => match *request {
+                    PoolManagementRequest::CheckIn(connection) => {
+                        self.check_in(*connection);
+                    }
+                    PoolManagementRequest::Clear {
+                        cause, service_id, ..
+                    } => {
+                        self.clear(cause, service_id);
+                    }
+                    PoolManagementRequest::MarkAsReady {
+                        _completion_handler,
+                    } => {
+                        self.mark_as_ready();
+                        _completion_handler.acknowledge(());
+                    }
+                    PoolManagementRequest::HandleConnectionSucceeded(conn) => {
+                        self.handle_connection_succeeded(conn);
+                    }
+                    PoolManagementRequest::HandleConnectionFailed => {
+                        self.handle_connection_failed();
+                    }
+                    #[cfg(test)]
+                    PoolManagementRequest::Sync(tx) => {
+                        let _ = tx.send(());
+                    }
+                },
                 PoolTask::Maintenance => {
                     self.perform_maintenance();
                 }
@@ -336,6 +340,10 @@ impl ConnectionPoolWorker {
         });
     }
 
+    fn below_max_connections(&self) -> bool {
+        self.total_connection_count < self.max_pool_size
+    }
+
     fn can_service_connection_request(&self) -> bool {
         if !matches!(self.state, PoolState::Ready) {
             return false;
@@ -345,8 +353,7 @@ impl ConnectionPoolWorker {
             return true;
         }
 
-        self.total_connection_count < self.max_pool_size
-            && self.pending_connection_count < MAX_CONNECTING
+        self.below_max_connections() && self.pending_connection_count < MAX_CONNECTING
     }
 
     async fn check_out(&mut self, request: ConnectionRequest) {
@@ -365,7 +372,7 @@ impl ConnectionPoolWorker {
             }
 
             conn.mark_as_in_use(self.manager.clone());
-            if let Err(request) = request.fulfill(ConnectionRequestResult::Pooled(conn)) {
+            if let Err(request) = request.fulfill(ConnectionRequestResult::Pooled(Box::new(conn))) {
                 // checking out thread stopped listening, indicating it hit the WaitQueue
                 // timeout, so we put connection back into pool.
                 let mut connection = request.unwrap_pooled_connection();
@@ -377,7 +384,7 @@ impl ConnectionPoolWorker {
         }
 
         // otherwise, attempt to create a connection.
-        if self.total_connection_count < self.max_pool_size {
+        if self.below_max_connections() {
             let event_handler = self.event_handler.clone();
             let establisher = self.establisher.clone();
             let pending_connection = self.create_pending_connection();
@@ -459,7 +466,8 @@ impl ConnectionPoolWorker {
             let count = self.service_connection_count.entry(sid).or_insert(0);
             *count += 1;
         }
-        if let ConnectionSucceeded::ForPool(mut connection) = connection {
+        if let ConnectionSucceeded::ForPool(connection) = connection {
+            let mut connection = *connection;
             connection.mark_as_available();
             self.available_connections.push_back(connection);
         }
@@ -503,12 +511,13 @@ impl ConnectionPoolWorker {
             self.emit_event(|handler| {
                 let event = PoolClearedEvent {
                     address: self.address.clone(),
+                    service_id,
                 };
 
                 handler.handle_pool_cleared_event(event);
             });
 
-            if !self.generation.is_load_balanced() {
+            if !matches!(self.generation, PoolGeneration::LoadBalanced(_)) {
                 for request in self.wait_queue.drain(..) {
                     // an error means the other end hung up already, which is okay because we were
                     // returning an error anyways
@@ -615,8 +624,9 @@ impl ConnectionPoolWorker {
                     .await;
 
                     if let Ok(connection) = connection {
-                        manager
-                            .handle_connection_succeeded(ConnectionSucceeded::ForPool(connection))
+                        manager.handle_connection_succeeded(ConnectionSucceeded::ForPool(Box::new(
+                            connection,
+                        )))
                     }
                 });
             }
@@ -641,6 +651,7 @@ async fn establish_connection(
 
     match establish_result {
         Err(ref e) => {
+            server_updater.handle_error(e.clone()).await;
             if let Some(handler) = event_handler {
                 let event = ConnectionClosedEvent {
                     address,
@@ -649,7 +660,6 @@ async fn establish_connection(
                 };
                 handler.handle_connection_closed_event(event);
             }
-            server_updater.handle_error(e.clone()).await;
             manager.handle_connection_failed();
         }
         Ok(ref mut connection) => {
@@ -682,7 +692,7 @@ enum PoolState {
 #[derive(Debug)]
 enum PoolTask {
     /// Handle a management request from a `PoolManager`.
-    HandleManagementRequest(PoolManagementRequest),
+    HandleManagementRequest(Box<PoolManagementRequest>),
 
     /// Fulfill the given connection request.
     CheckOut(ConnectionRequest),
@@ -693,7 +703,7 @@ enum PoolTask {
 
 impl From<PoolManagementRequest> for PoolTask {
     fn from(request: PoolManagementRequest) -> Self {
-        PoolTask::HandleManagementRequest(request)
+        PoolTask::HandleManagementRequest(Box::new(request))
     }
 }
 
@@ -701,14 +711,17 @@ impl From<PoolManagementRequest> for PoolTask {
 /// to it.
 fn handle_channel() -> (PoolWorkerHandle, HandleListener) {
     let (sender, receiver) = mpsc::channel(1);
-    (PoolWorkerHandle { sender }, HandleListener { receiver })
+    (
+        PoolWorkerHandle { _sender: sender },
+        HandleListener { receiver },
+    )
 }
 
 /// Handle to the worker. Once all handles have been dropped, the worker
 /// will stop waiting for new requests and drop the pool itself.
 #[derive(Debug, Clone)]
 pub(super) struct PoolWorkerHandle {
-    sender: mpsc::Sender<()>,
+    _sender: mpsc::Sender<()>,
 }
 
 impl PoolWorkerHandle {
@@ -749,10 +762,6 @@ impl PoolGeneration {
         Self::LoadBalanced(HashMap::new())
     }
 
-    fn is_load_balanced(&self) -> bool {
-        matches!(self, Self::LoadBalanced(_))
-    }
-
     #[cfg(test)]
     pub(crate) fn as_normal(&self) -> Option<u32> {
         match self {
