@@ -18,6 +18,7 @@ use crate::{
     bson::{doc, to_document, Bson, Document},
     bson_util,
     client::session::TransactionState,
+    cmap::conn::PinnedConnectionHandle,
     concern::{ReadConcern, WriteConcern},
     error::{convert_bulk_errors, BulkWriteError, BulkWriteFailure, Error, ErrorKind, Result},
     index::IndexModel,
@@ -111,12 +112,23 @@ use crate::{
 /// # }
 /// ```
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct Collection<T> {
     inner: Arc<CollectionInner>,
     _phantom: std::marker::PhantomData<T>,
 }
 
+// Because derive is too conservative, derive only implements Clone if T is Clone.
+// Collection<T> does not actually store any value of type T (so T does not need to be clone).
+impl<T> Clone for Collection<T> {
+    fn clone(&self) -> Self {
+        Self {
+            inner: self.inner.clone(),
+            _phantom: Default::default(),
+        }
+    }
+}
+
 #[derive(Debug)]
 struct CollectionInner {
     client: Client,
@@ -155,13 +167,10 @@ impl<T> Collection<T> {
 
     /// Gets a clone of the `Collection` with a different type `U`.
     pub fn clone_with_type<U>(&self) -> Collection<U> {
-        let options = CollectionOptions::builder()
-            .selection_criteria(self.inner.selection_criteria.clone())
-            .read_concern(self.inner.read_concern.clone())
-            .write_concern(self.inner.write_concern.clone())
-            .build();
-
-        Collection::new(self.inner.db.clone(), &self.inner.name, Some(options))
+        Collection {
+            inner: self.inner.clone(),
+            _phantom: Default::default(),
+        }
     }
 
     /// Get the `Client` that this collection descended from.
@@ -249,10 +258,7 @@ impl<T> Collection<T> {
 
         let aggregate = Aggregate::new(self.namespace(), pipeline, options);
         let client = self.client();
-        client
-            .execute_cursor_operation(aggregate)
-            .await
-            .map(|(spec, session)| Cursor::new(client.clone(), spec, session))
+        client.execute_cursor_operation(aggregate).await
     }
 
     /// Runs an aggregation operation using the provided `ClientSession`.
@@ -273,9 +279,8 @@ impl<T> Collection<T> {
         let aggregate = Aggregate::new(self.namespace(), pipeline, options);
         let client = self.client();
         client
-            .execute_operation(aggregate, session)
+            .execute_session_cursor_operation(aggregate, session)
             .await
-            .map(|result| SessionCursor::new(client.clone(), result))
     }
 
     /// Estimates the number of documents in the collection using collection metadata.
@@ -602,10 +607,7 @@ impl<T> Collection<T> {
     ) -> Result<Cursor<IndexModel>> {
         let list_indexes = ListIndexes::new(self.namespace(), options.into());
         let client = self.client();
-        client
-            .execute_cursor_operation(list_indexes)
-            .await
-            .map(|(spec, session)| Cursor::new(client.clone(), spec, session))
+        client.execute_cursor_operation(list_indexes).await
     }
 
     /// Lists all indexes on this collection using the provided `ClientSession`.
@@ -617,9 +619,8 @@ impl<T> Collection<T> {
         let list_indexes = ListIndexes::new(self.namespace(), options.into());
         let client = self.client();
         client
-            .execute_operation(list_indexes, session)
+            .execute_session_cursor_operation(list_indexes, session)
             .await
-            .map(|spec| SessionCursor::new(client.clone(), spec))
     }
 
     async fn list_index_names_common(
@@ -766,17 +767,23 @@ impl<T> Collection<T> {
     }
 
     /// Kill the server side cursor that id corresponds to.
-    pub(super) async fn kill_cursor(&self, cursor_id: i64) -> Result<()> {
+    pub(super) async fn kill_cursor(
+        &self,
+        cursor_id: i64,
+        pinned_connection: Option<&PinnedConnectionHandle>,
+    ) -> Result<()> {
         let ns = self.namespace();
 
         self.client()
             .database(ns.db.as_str())
-            .run_command(
+            .run_command_common(
                 doc! {
                     "killCursors": ns.coll.as_str(),
                     "cursors": [cursor_id]
                 },
                 None,
+                None,
+                pinned_connection,
             )
             .await?;
         Ok(())
@@ -799,10 +806,7 @@ where
         let find = Find::<T>::new(self.namespace(), filter.into(), options);
         let client = self.client();
 
-        client
-            .execute_cursor_operation(find)
-            .await
-            .map(|(result, session)| Cursor::new(client.clone(), result, session))
+        client.execute_cursor_operation(find).await
     }
 
     /// Finds the documents in the collection matching `filter` using the provided `ClientSession`.
@@ -819,10 +823,7 @@ where
         let find = Find::<T>::new(self.namespace(), filter.into(), options);
         let client = self.client();
 
-        client
-            .execute_operation(find, session)
-            .await
-            .map(|result| SessionCursor::new(client.clone(), result))
+        client.execute_session_cursor_operation(find, session).await
     }
 
     /// Finds a single document in the collection matching `filter`.
@@ -1077,46 +1078,51 @@ where
 
                     n_attempted += current_batch_size;
                 }
-                Err(e) => match *e.kind {
-                    ErrorKind::BulkWrite(bw) => {
-                        // for ordered inserts this size will be incorrect, but knowing the batch
-                        // size isn't needed for ordered failures since we
-                        // return immediately from them anyways.
-                        let current_batch_size = bw.inserted_ids.len()
-                            + bw.write_errors.as_ref().map(|we| we.len()).unwrap_or(0);
+                Err(e) => {
+                    let labels = e.labels().clone();
+                    match *e.kind {
+                        ErrorKind::BulkWrite(bw) => {
+                            // for ordered inserts this size will be incorrect, but knowing the
+                            // batch size isn't needed for ordered
+                            // failures since we return immediately from
+                            // them anyways.
+                            let current_batch_size = bw.inserted_ids.len()
+                                + bw.write_errors.as_ref().map(|we| we.len()).unwrap_or(0);
 
-                        let failure_ref =
-                            cumulative_failure.get_or_insert_with(BulkWriteFailure::new);
-                        if let Some(write_errors) = bw.write_errors {
-                            for err in write_errors {
-                                let index = n_attempted + err.index;
+                            let failure_ref =
+                                cumulative_failure.get_or_insert_with(BulkWriteFailure::new);
+                            if let Some(write_errors) = bw.write_errors {
+                                for err in write_errors {
+                                    let index = n_attempted + err.index;
 
-                                failure_ref
-                                    .write_errors
-                                    .get_or_insert_with(Default::default)
-                                    .push(BulkWriteError { index, ..err });
+                                    failure_ref
+                                        .write_errors
+                                        .get_or_insert_with(Default::default)
+                                        .push(BulkWriteError { index, ..err });
+                                }
                             }
-                        }
 
-                        if let Some(wc_error) = bw.write_concern_error {
-                            failure_ref.write_concern_error = Some(wc_error);
-                        }
-
-                        error_labels.extend(e.labels);
-
-                        if ordered {
-                            // this will always be true since we invoked get_or_insert_with above.
-                            if let Some(failure) = cumulative_failure {
-                                return Err(Error {
-                                    kind: Box::new(ErrorKind::BulkWrite(failure)),
-                                    labels: error_labels,
-                                });
+                            if let Some(wc_error) = bw.write_concern_error {
+                                failure_ref.write_concern_error = Some(wc_error);
                             }
+
+                            error_labels.extend(labels);
+
+                            if ordered {
+                                // this will always be true since we invoked get_or_insert_with
+                                // above.
+                                if let Some(failure) = cumulative_failure {
+                                    return Err(Error::new(
+                                        ErrorKind::BulkWrite(failure),
+                                        Some(error_labels),
+                                    ));
+                                }
+                            }
+                            n_attempted += current_batch_size;
                         }
-                        n_attempted += current_batch_size;
+                        _ => return Err(e),
                     }
-                    _ => return Err(e),
-                },
+                }
             }
         }
 
@@ -1131,6 +1137,9 @@ where
 
     /// Inserts the data in `docs` into the collection.
     ///
+    /// Note that this method accepts both owned and borrowed values, so the input documents
+    /// do not need to be cloned in order to be passed in.
+    ///
     /// This operation will retry once upon failure if the connection and encountered error support
     /// retryability. See the documentation
     /// [here](https://docs.mongodb.com/manual/core/retryable-writes/) for more information on
@@ -1145,6 +1154,9 @@ where
 
     /// Inserts the data in `docs` into the collection using the provided `ClientSession`.
     ///
+    /// Note that this method accepts both owned and borrowed values, so the input documents
+    /// do not need to be cloned in order to be passed in.
+    ///
     /// This operation will retry once upon failure if the connection and encountered error support
     /// retryability. See the documentation
     /// [here](https://docs.mongodb.com/manual/core/retryable-writes/) for more information on
@@ -1183,6 +1195,9 @@ where
 
     /// Inserts `doc` into the collection.
     ///
+    /// Note that either an owned or borrowed value can be inserted here, so the input document
+    /// does not need to be cloned to be passed in.
+    ///
     /// This operation will retry once upon failure if the connection and encountered error support
     /// retryability. See the documentation
     /// [here](https://docs.mongodb.com/manual/core/retryable-writes/) for more information on
@@ -1197,6 +1212,9 @@ where
 
     /// Inserts `doc` into the collection using the provided `ClientSession`.
     ///
+    /// Note that either an owned or borrowed value can be inserted here, so the input document
+    /// does not need to be cloned to be passed in.
+    ///
     /// This operation will retry once upon failure if the connection and encountered error support
     /// retryability. See the documentation
     /// [here](https://docs.mongodb.com/manual/core/retryable-writes/) for more information on
