@@ -27,9 +27,24 @@ use crate::{
 pub struct ReadConcern {
     /// The level of the read concern.
     pub level: ReadConcernLevel,
+}
+
+/// An internal-only read concern type that allows the omission of a "level" as well as
+/// specification of "atClusterTime" and "afterClusterTime".
+#[skip_serializing_none]
+#[derive(Clone, Debug, Deserialize, Serialize, PartialEq)]
+#[serde(rename_all = "camelCase")]
+#[serde(rename = "readConcern")]
+pub(crate) struct ReadConcernInternal {
+    /// The level of the read concern.
+    pub(crate) level: Option<ReadConcernLevel>,
 
     /// The snapshot read timestamp.
     pub(crate) at_cluster_time: Option<Timestamp>,
+
+    /// The time of most recent operation using this session.
+    /// Used for providing causal consistency.
+    pub(crate) after_cluster_time: Option<Timestamp>,
 }
 
 impl ReadConcern {
@@ -90,12 +105,19 @@ impl ReadConcern {
     }
 }
 
+impl From<ReadConcern> for ReadConcernInternal {
+    fn from(rc: ReadConcern) -> Self {
+        ReadConcernInternal {
+            level: Some(rc.level),
+            at_cluster_time: None,
+            after_cluster_time: None,
+        }
+    }
+}
+
 impl From<ReadConcernLevel> for ReadConcern {
     fn from(level: ReadConcernLevel) -> Self {
-        Self {
-            level,
-            at_cluster_time: None,
-        }
+        Self { level }
     }
 }
 
