@@ -8,18 +8,19 @@ use std::{
 
 use futures_core::{future::BoxFuture, Stream};
 use serde::de::DeserializeOwned;
+#[cfg(test)]
+use tokio::sync::oneshot;
 
 use crate::{
-    bson::Document,
+    cmap::conn::PinnedConnectionHandle,
     error::{Error, Result},
     operation::GetMore,
     results::GetMoreResult,
     Client,
     ClientSession,
-    RUNTIME,
 };
-pub(crate) use common::{CursorInformation, CursorSpecification};
-use common::{GenericCursor, GetMoreProvider, GetMoreProviderResult};
+use common::{kill_cursor, GenericCursor, GetMoreProvider, GetMoreProviderResult};
+pub(crate) use common::{CursorInformation, CursorSpecification, PinnedConnection};
 
 /// A [`Cursor`] streams the result of a query. When a query is made, the returned [`Cursor`] will
 /// contain the first batch of results from the server; the individual results will then be returned
@@ -85,6 +86,8 @@ where
 {
     client: Client,
     wrapped_cursor: ImplicitSessionCursor<T>,
+    #[cfg(test)]
+    kill_watcher: Option<oneshot::Sender<()>>,
     _phantom: std::marker::PhantomData<T>,
 }
 
@@ -96,15 +99,37 @@ where
         client: Client,
         spec: CursorSpecification<T>,
         session: Option<ClientSession>,
+        pin: Option<PinnedConnectionHandle>,
     ) -> Self {
         let provider = ImplicitSessionGetMoreProvider::new(&spec, session);
 
         Self {
             client: client.clone(),
-            wrapped_cursor: ImplicitSessionCursor::new(client, spec, provider),
+            wrapped_cursor: ImplicitSessionCursor::new(
+                client,
+                spec,
+                PinnedConnection::new(pin),
+                provider,
+            ),
+            #[cfg(test)]
+            kill_watcher: None,
             _phantom: Default::default(),
         }
     }
+
+    /// Some tests need to be able to observe the events generated by `killCommand` execution;
+    /// however, because that happens asynchronously on `drop`, the test runner can conclude before
+    /// the event is published.  To fix that, tests can set a "kill watcher" on cursors - a
+    /// one-shot channel with a `()` value pushed after `killCommand` is run that the test can wait
+    /// on.
+    #[cfg(test)]
+    pub(crate) fn set_kill_watcher(&mut self, tx: oneshot::Sender<()>) {
+        assert!(
+            self.kill_watcher.is_none(),
+            "cursor already has a kill_watcher"
+        );
+        self.kill_watcher = Some(tx);
+    }
 }
 
 impl<T> Stream for Cursor<T>
@@ -127,13 +152,14 @@ where
             return;
         }
 
-        let ns = self.wrapped_cursor.namespace();
-        let coll = self
-            .client
-            .database(ns.db.as_str())
-            .collection::<Document>(ns.coll.as_str());
-        let cursor_id = self.wrapped_cursor.id();
-        RUNTIME.execute(async move { coll.kill_cursor(cursor_id).await });
+        kill_cursor(
+            self.client.clone(),
+            self.wrapped_cursor.namespace(),
+            self.wrapped_cursor.id(),
+            self.wrapped_cursor.pinned_connection().replicate(),
+            #[cfg(test)]
+            self.kill_watcher.take(),
+        );
     }
 }
 
@@ -185,7 +211,7 @@ impl<T: Send + Sync + DeserializeOwned> GetMoreProvider for ImplicitSessionGetMo
     fn executing_future(&mut self) -> Option<&mut Self::GetMoreFuture> {
         match self {
             Self::Executing(ref mut future) => Some(future),
-            Self::Idle(_) | Self::Done => None,
+            Self::Idle { .. } | Self::Done => None,
         }
     }
 
@@ -194,15 +220,21 @@ impl<T: Send + Sync + DeserializeOwned> GetMoreProvider for ImplicitSessionGetMo
         if exhausted {
             *self = Self::Done;
         } else {
-            *self = Self::Idle(session)
+            *self = Self::Idle(session);
         }
     }
 
-    fn start_execution(&mut self, info: CursorInformation, client: Client) {
+    fn start_execution(
+        &mut self,
+        info: CursorInformation,
+        client: Client,
+        pinned_connection: Option<&PinnedConnectionHandle>,
+    ) {
         take_mut::take(self, |self_| match self_ {
             Self::Idle(mut session) => {
+                let pinned_connection = pinned_connection.map(|c| c.replicate());
                 let future = Box::pin(async move {
-                    let get_more = GetMore::new(info);
+                    let get_more = GetMore::new(info, pinned_connection.as_ref());
                     let get_more_result = client
                         .execute_operation(get_more, session.as_mut().map(|b| b.as_mut()))
                         .await;
