@@ -7,17 +7,26 @@ use std::{
 use futures_core::{future::BoxFuture, Stream};
 use futures_util::StreamExt;
 use serde::de::DeserializeOwned;
+#[cfg(test)]
+use tokio::sync::oneshot;
 
-use super::common::{CursorInformation, GenericCursor, GetMoreProvider, GetMoreProviderResult};
+use super::common::{
+    kill_cursor,
+    CursorInformation,
+    GenericCursor,
+    GetMoreProvider,
+    GetMoreProviderResult,
+    PinnedConnection,
+};
 use crate::{
     bson::Document,
+    cmap::conn::PinnedConnectionHandle,
     cursor::CursorSpecification,
     error::{Error, Result},
     operation::GetMore,
     results::GetMoreResult,
     Client,
     ClientSession,
-    RUNTIME,
 };
 
 /// A [`SessionCursor`] is a cursor that was created with a [`ClientSession`] that must be iterated
@@ -55,17 +64,27 @@ where
     client: Client,
     info: CursorInformation,
     buffer: VecDeque<T>,
+    pinned_connection: PinnedConnection,
+    #[cfg(test)]
+    kill_watcher: Option<oneshot::Sender<()>>,
 }
 
 impl<T> SessionCursor<T>
 where
     T: DeserializeOwned + Unpin + Send + Sync,
 {
-    pub(crate) fn new(client: Client, spec: CursorSpecification<T>) -> Self {
+    pub(crate) fn new(
+        client: Client,
+        spec: CursorSpecification<T>,
+        pinned: Option<PinnedConnectionHandle>,
+    ) -> Self {
         Self {
             client,
             info: spec.info,
             buffer: spec.initial_buffer,
+            pinned_connection: PinnedConnection::new(pinned),
+            #[cfg(test)]
+            kill_watcher: None,
         }
     }
 
@@ -127,6 +146,7 @@ where
             generic_cursor: ExplicitSessionCursor::new(
                 self.client.clone(),
                 spec,
+                self.pinned_connection.replicate(),
                 get_more_provider,
             ),
             session_cursor: self,
@@ -159,6 +179,20 @@ where
     pub async fn next(&mut self, session: &mut ClientSession) -> Option<Result<T>> {
         self.stream(session).next().await
     }
+
+    /// Some tests need to be able to observe the events generated by `killCommand` execution;
+    /// however, because that happens asynchronously on `drop`, the test runner can conclude before
+    /// the event is published.  To fix that, tests can set a "kill watcher" on cursors - a
+    /// one-shot channel with a `()` value pushed after `killCommand` is run that the test can wait
+    /// on.
+    #[cfg(test)]
+    pub(crate) fn set_kill_watcher(&mut self, tx: oneshot::Sender<()>) {
+        assert!(
+            self.kill_watcher.is_none(),
+            "cursor already has a kill_watcher"
+        );
+        self.kill_watcher = Some(tx);
+    }
 }
 
 impl<T> SessionCursor<T>
@@ -183,13 +217,14 @@ where
             return;
         }
 
-        let ns = &self.info.ns;
-        let coll = self
-            .client
-            .database(ns.db.as_str())
-            .collection::<Document>(ns.coll.as_str());
-        let cursor_id = self.info.id;
-        RUNTIME.execute(async move { coll.kill_cursor(cursor_id).await });
+        kill_cursor(
+            self.client.clone(),
+            &self.info.ns,
+            self.info.id,
+            self.pinned_connection.replicate(),
+            #[cfg(test)]
+            self.kill_watcher.take(),
+        );
     }
 }
 
@@ -266,7 +301,7 @@ impl<'session, T: Send + Sync + DeserializeOwned> GetMoreProvider
     fn executing_future(&mut self) -> Option<&mut Self::GetMoreFuture> {
         match self {
             Self::Executing(future) => Some(future),
-            Self::Idle(_) => None,
+            Self::Idle { .. } => None,
         }
     }
 
@@ -274,11 +309,17 @@ impl<'session, T: Send + Sync + DeserializeOwned> GetMoreProvider
         *self = Self::Idle(MutableSessionReference { reference: session })
     }
 
-    fn start_execution(&mut self, info: CursorInformation, client: Client) {
+    fn start_execution(
+        &mut self,
+        info: CursorInformation,
+        client: Client,
+        pinned_connection: Option<&PinnedConnectionHandle>,
+    ) {
         take_mut::take(self, |self_| {
             if let ExplicitSessionGetMoreProvider::Idle(session) = self_ {
+                let pinned_connection = pinned_connection.map(|c| c.replicate());
                 let future = Box::pin(async move {
-                    let get_more = GetMore::new(info);
+                    let get_more = GetMore::new(info, pinned_connection.as_ref());
                     let get_more_result = client
                         .execute_operation(get_more, Some(&mut *session.reference))
                         .await;
