@@ -13,9 +13,7 @@ use crate::{
     selection_criteria::SelectionCriteria,
 };
 
-use super::CommandResponse;
-
-const SERVER_4_9_0_WIRE_VERSION: i32 = 12;
+use super::{CommandResponse, SERVER_4_9_0_WIRE_VERSION};
 
 pub(crate) struct Count {
     ns: Namespace,
@@ -47,8 +45,10 @@ impl Operation for Count {
     const NAME: &'static str = "count";
 
     fn build(&mut self, description: &StreamDescription) -> Result<Command> {
+        let mut name = Self::NAME.to_string();
         let mut body = match description.max_wire_version {
             Some(v) if v >= SERVER_4_9_0_WIRE_VERSION => {
+                name = "aggregate".to_string();
                 doc! {
                     "aggregate": self.ns.coll.clone(),
                     "pipeline": [
@@ -74,9 +74,10 @@ impl Operation for Count {
 
         append_options(&mut body, self.options.as_ref())?;
 
-        Ok(Command::new(
-            Self::NAME.to_string(),
+        Ok(Command::new_read(
+            name,
             self.ns.db.clone(),
+            self.options.as_ref().and_then(|o| o.read_concern.clone()),
             body,
         ))
     }
@@ -87,10 +88,9 @@ impl Operation for Count {
         description: &StreamDescription,
     ) -> Result<Self::O> {
         let response_body: ResponseBody = match (description.max_wire_version, response) {
-            (Some(v), Response::Aggregate(CursorBody { mut cursor, .. }))
-                if v >= SERVER_4_9_0_WIRE_VERSION =>
-            {
-                cursor
+            (Some(v), Response::Aggregate(mut cursor_body)) if v >= SERVER_4_9_0_WIRE_VERSION => {
+                cursor_body
+                    .cursor
                     .first_batch
                     .pop_front()
                     .and_then(|doc| bson::from_document(doc).ok())
@@ -127,6 +127,10 @@ impl Operation for Count {
         None
     }
 
+    fn supports_read_concern(&self, _description: &StreamDescription) -> bool {
+        true
+    }
+
     fn retryability(&self) -> Retryability {
         Retryability::Read
     }
@@ -135,7 +139,7 @@ impl Operation for Count {
 #[derive(Debug, Deserialize)]
 #[serde(untagged)]
 pub(crate) enum Response {
-    Aggregate(CursorBody<Document>),
+    Aggregate(Box<CursorBody<Document>>),
     Count(ResponseBody),
 }
 
