@@ -150,11 +150,11 @@ impl AsyncStream {
         // If there are TLS options, wrap the inner stream with rustls.
         match options.tls_options {
             Some(cfg) => {
-                let name = DNSNameRef::try_from_ascii_str(options.address.host()).map_err(|e| {
-                    ErrorKind::DnsResolve {
-                        message: e.to_string(),
-                    }
-                })?;
+                let host = options.address.host();
+                let name =
+                    DNSNameRef::try_from_ascii_str(host).map_err(|e| ErrorKind::DnsResolve {
+                        message: format!("could not resolve {:?}: {}", host, e.to_string()),
+                    })?;
                 let mut tls_config = cfg.into_rustls_config()?;
                 tls_config.enable_sni = true;
 
