@@ -12,24 +12,43 @@ use crate::{
 
 const DRIVER_MIN_DB_VERSION: &str = "3.6";
 const DRIVER_MIN_WIRE_VERSION: i32 = 6;
-const DRIVER_MAX_WIRE_VERSION: i32 = 13;
+const DRIVER_MAX_WIRE_VERSION: i32 = 14;
 
+/// Enum representing the possible types of servers that the driver can connect to.
 #[derive(Debug, Deserialize, Clone, Copy, Eq, PartialEq)]
 #[non_exhaustive]
 pub enum ServerType {
+    /// A single, non-replica set mongod.
     Standalone,
+
+    /// A router used in sharded deployments.
     Mongos,
+
+    /// The primary node in a replica set.
     #[serde(rename = "RSPrimary")]
     RsPrimary,
+
+    /// A secondary node in a replica set.
     #[serde(rename = "RSSecondary")]
     RsSecondary,
+
+    /// A non-data bearing node in a replica set which can participate in elections.
     #[serde(rename = "RSArbiter")]
     RsArbiter,
+
+    /// Hidden, starting up, or recovering nodes in a replica set.
     #[serde(rename = "RSOther")]
     RsOther,
+
+    /// A member of an uninitialized replica set or a member that has been removed from the replica
+    /// set config.
     #[serde(rename = "RSGhost")]
     RsGhost,
+
+    /// A load-balancing proxy between the driver and the MongoDB deployment.
     LoadBalancer,
+
+    /// A server that the driver hasn't yet communicated with or can't connect to.
     #[serde(alias = "PossiblePrimary")]
     Unknown,
 }
@@ -177,20 +196,6 @@ impl ServerDescription {
         description
     }
 
-    pub(crate) fn new_load_balancer(mut address: ServerAddress) -> Self {
-        address = ServerAddress::Tcp {
-            host: address.host().to_lowercase(),
-            port: address.port(),
-        };
-        Self {
-            address,
-            server_type: ServerType::LoadBalancer,
-            last_update_time: None,
-            reply: Ok(None),
-            average_round_trip_time: None,
-        }
-    }
-
     /// Whether this server is "available" as per the definition in the server selection spec.
     pub(crate) fn is_available(&self) -> bool {
         self.server_type.is_available()
