@@ -65,8 +65,7 @@ async fn run_test(test_file: TestFile) {
 
     let topology_description = test_file
         .topology_description
-        .into_topology_description(None)
-        .unwrap();
+        .into_topology_description(None);
 
     let read_pref = ReadPreference::Nearest {
         options: Default::default(),
@@ -121,6 +120,11 @@ async fn load_balancing_test() {
         return;
     }
 
+    if setup_client_options.load_balanced.unwrap_or(false) {
+        println!("skipping load_balancing_test test due to load-balanced topology");
+        return;
+    }
+
     setup_client_options.hosts.drain(1..);
     setup_client_options.direct_connection = Some(true);
     let setup_client = TestClient::with_options(Some(setup_client_options)).await;
