@@ -10,6 +10,9 @@ use crate::{
 use super::{TestServerDescription, TestTopologyDescription};
 
 #[derive(Debug, Deserialize)]
+// TODO RUST-1080: remove the #[allow(dead_code)] tag and add #[serde(deny_unknown_fields)] to
+// ensure these tests are being fully run
+#[allow(dead_code)]
 struct TestFile {
     #[serde(rename = "heartbeatFrequencyMS")]
     heartbeat_frequency_ms: Option<u64>,
@@ -63,13 +66,9 @@ async fn run_test(test_file: TestFile) {
         None => return,
     };
 
-    let topology = match test_file
+    let topology = test_file
         .topology_description
-        .into_topology_description(test_file.heartbeat_frequency_ms.map(Duration::from_millis))
-    {
-        Some(t) => t,
-        None => return,
-    };
+        .into_topology_description(test_file.heartbeat_frequency_ms.map(Duration::from_millis));
 
     if let Some(ref expected_suitable_servers) = test_file.suitable_servers {
         let mut actual_servers: Vec<_> = topology.suitable_servers(&read_pref).unwrap();
@@ -150,6 +149,21 @@ async fn server_selection_unknown() {
     .await;
 }
 
+#[cfg_attr(feature = "tokio-runtime", tokio::test)]
+#[cfg_attr(feature = "async-std-runtime", async_std::test)]
+async fn server_selection_load_balanced() {
+    run_spec_test(
+        &[
+            "server-selection",
+            "server_selection",
+            "LoadBalanced",
+            "read",
+        ],
+        run_test,
+    )
+    .await;
+}
+
 #[cfg_attr(feature = "tokio-runtime", tokio::test)]
 #[cfg_attr(feature = "async-std-runtime", async_std::test)]
 async fn max_staleness_replica_set_no_primary() {
