@@ -1,4 +1,4 @@
-use std::time::Duration;
+use std::{collections::HashMap, sync::Arc, time::Duration};
 
 use serde::Deserialize;
 
@@ -12,7 +12,7 @@ use crate::{
         ServerType,
         TopologyDescription,
     },
-    selection_criteria::TagSet,
+    selection_criteria::{SelectionCriteria, TagSet},
 };
 
 mod in_window;
@@ -29,21 +29,16 @@ impl TestTopologyDescription {
     fn into_topology_description(
         self,
         heartbeat_frequency: Option<Duration>,
-    ) -> Option<TopologyDescription> {
-        let servers: Option<Vec<ServerDescription>> = self
+    ) -> TopologyDescription {
+        let servers: HashMap<ServerAddress, ServerDescription> = self
             .servers
             .into_iter()
-        // The driver doesn't support server versions low enough not to support max staleness, so we
-        // just manually filter them out here.
-            .filter(|server| server.max_wire_version.map(|version| version >= 5).unwrap_or(true))
-            .map(|sd| sd.into_server_description())
+            .filter_map(|sd| {
+                sd.into_server_description()
+                    .map(|sd| (sd.address.clone(), sd))
+            })
             .collect();
 
-        let servers = match servers {
-            Some(servers) => servers,
-            None => return None,
-        };
-
         TopologyDescription {
             single_seed: servers.len() == 1,
             topology_type: self.topology_type,
@@ -56,17 +51,13 @@ impl TestTopologyDescription {
             cluster_time: None,
             local_threshold: None,
             heartbeat_freq: heartbeat_frequency,
-            servers: servers
-                .into_iter()
-                .map(|server| (server.address.clone(), server))
-                .collect(),
+            servers,
         }
-        .into()
     }
 }
 
 #[derive(Debug, Deserialize)]
-#[serde(rename_all = "camelCase")]
+#[serde(rename_all = "camelCase", deny_unknown_fields)]
 struct TestServerDescription {
     address: String,
     #[serde(rename = "avg_rtt_ms")]
@@ -76,7 +67,9 @@ struct TestServerDescription {
     tags: Option<TagSet>,
     last_update_time: Option<i32>,
     last_write: Option<LastWriteDate>,
-    max_wire_version: Option<i32>,
+    // We don't need to use this field, but it needs to be included during deserialization so that
+    // we can use the deny_unknown_fields tag.
+    _max_wire_version: Option<i32>,
 }
 
 impl TestServerDescription {
@@ -134,6 +127,7 @@ enum TestServerType {
     RsOther,
     #[serde(rename = "RSGhost")]
     RsGhost,
+    LoadBalancer,
     Unknown,
     PossiblePrimary,
 }
@@ -148,6 +142,7 @@ impl TestServerType {
             TestServerType::RsArbiter => Some(ServerType::RsArbiter),
             TestServerType::RsOther => Some(ServerType::RsOther),
             TestServerType::RsGhost => Some(ServerType::RsGhost),
+            TestServerType::LoadBalancer => Some(ServerType::LoadBalancer),
             TestServerType::Unknown => Some(ServerType::Unknown),
             TestServerType::PossiblePrimary => None,
         }
@@ -188,3 +183,66 @@ fn is_master_response_from_server_type(server_type: ServerType) -> Option<IsMast
 
     Some(response)
 }
+
+#[test]
+fn predicate_omits_unavailable() {
+    let criteria = SelectionCriteria::Predicate(Arc::new(|si| {
+        !matches!(si.server_type(), ServerType::RsPrimary)
+    }));
+
+    let desc = TestTopologyDescription {
+        topology_type: TopologyType::ReplicaSetWithPrimary,
+        servers: vec![
+            TestServerDescription {
+                address: "localhost:27017".to_string(),
+                avg_rtt_ms: Some(12.0),
+                server_type: TestServerType::RsPrimary,
+                tags: None,
+                last_update_time: None,
+                last_write: None,
+                _max_wire_version: None,
+            },
+            TestServerDescription {
+                address: "localhost:27018".to_string(),
+                avg_rtt_ms: Some(12.0),
+                server_type: TestServerType::Unknown,
+                tags: None,
+                last_update_time: None,
+                last_write: None,
+                _max_wire_version: None,
+            },
+            TestServerDescription {
+                address: "localhost:27019".to_string(),
+                avg_rtt_ms: Some(12.0),
+                server_type: TestServerType::RsArbiter,
+                tags: None,
+                last_update_time: None,
+                last_write: None,
+                _max_wire_version: None,
+            },
+            TestServerDescription {
+                address: "localhost:27020".to_string(),
+                avg_rtt_ms: Some(12.0),
+                server_type: TestServerType::RsGhost,
+                tags: None,
+                last_update_time: None,
+                last_write: None,
+                _max_wire_version: None,
+            },
+            TestServerDescription {
+                address: "localhost:27021".to_string(),
+                avg_rtt_ms: Some(12.0),
+                server_type: TestServerType::RsOther,
+                tags: None,
+                last_update_time: None,
+                last_write: None,
+                _max_wire_version: None,
+            },
+        ],
+    }
+    .into_topology_description(None);
+    pretty_assertions::assert_eq!(
+        desc.suitable_servers_in_latency_window(&criteria).unwrap(),
+        Vec::<&ServerDescription>::new()
+    );
+}
