@@ -139,7 +139,9 @@ impl HeartbeatMonitor {
             #[cfg(test)]
             let min_frequency = self
                 .client_options
-                .heartbeat_freq_test
+                .test_options
+                .as_ref()
+                .and_then(|to| to.heartbeat_freq)
                 .unwrap_or(MIN_HEARTBEAT_FREQUENCY);
 
             #[cfg(not(test))]
@@ -253,13 +255,20 @@ impl UpdateMonitor {
                 _ => return,
             };
 
-            match update.into_message() {
+            // This needs to borrow the message rather than taking it so the update isn't sent
+            // until after the topology has processed the error.
+            match update.message() {
                 ServerUpdate::Error { error } => {
                     topology
-                        .handle_application_error(error.cause, error.handshake_phase, &server)
+                        .handle_application_error(
+                            error.cause.clone(),
+                            error.handshake_phase.clone(),
+                            &server,
+                        )
                         .await;
                 }
             }
+            drop(update);
         }
     }
 }
