@@ -7,8 +7,11 @@ use std::{
         Arc,
         Weak,
     },
+    time::Duration,
 };
 
+#[cfg(test)]
+use futures_util::stream::{FuturesUnordered, StreamExt};
 use tokio::sync::{RwLock, RwLockWriteGuard};
 
 use self::server::Server;
@@ -78,89 +81,13 @@ struct TopologyState {
     http_client: HttpClient,
     description: TopologyDescription,
     servers: HashMap<ServerAddress, Arc<Server>>,
-    #[cfg(test)]
-    mocked: bool,
     options: ClientOptions,
     id: ObjectId,
 }
 
 impl Topology {
-    /// Creates a new TopologyDescription with the set of servers initialized to the addresses
-    /// specified in `hosts` and each other field set to its default value. No monitoring threads
-    /// will be started for the servers in the topology that's returned.
-    #[cfg(test)]
-    pub(super) fn new_mocked(options: ClientOptions) -> Self {
-        let description = TopologyDescription::new(options.clone()).unwrap();
-
-        let id = ObjectId::new();
-        if let Some(ref handler) = options.sdam_event_handler {
-            let event = TopologyOpeningEvent { topology_id: id };
-            handler.handle_topology_opening_event(event);
-        }
-
-        let common = Common {
-            is_alive: Arc::new(AtomicBool::new(true)),
-            message_manager: TopologyMessageManager::new(),
-            options: options.clone(),
-            id,
-        };
-
-        let http_client = HttpClient::default();
-
-        let state = TopologyState {
-            description,
-            servers: Default::default(),
-            http_client: http_client.clone(),
-            mocked: true,
-            id,
-            options: options.clone(),
-        };
-
-        let topology = Self {
-            state: Arc::new(RwLock::new(state)),
-            common,
-        };
-
-        // we can block in place here because we're the only ones with access to the lock, so it
-        // should be acquired immediately.
-        let mut topology_state = RUNTIME.block_in_place(topology.state.write());
-
-        for address in &options.hosts {
-            topology_state.servers.insert(
-                address.clone(),
-                Server::create(
-                    address.clone(),
-                    &options,
-                    topology.downgrade(),
-                    http_client.clone(),
-                )
-                .0,
-            );
-        }
-
-        if let Some(ref handler) = options.sdam_event_handler {
-            let event = TopologyDescriptionChangedEvent {
-                topology_id: id,
-                previous_description: TopologyDescription::new_empty().into(),
-                new_description: topology_state.description.clone().into(),
-            };
-            handler.handle_topology_description_changed_event(event);
-
-            for server_address in &options.hosts {
-                let event = ServerOpeningEvent {
-                    topology_id: id,
-                    address: server_address.clone(),
-                };
-                handler.handle_server_opening_event(event);
-            }
-        }
-
-        drop(topology_state);
-        topology
-    }
-
     /// Creates a new Topology given the `options`.
-    pub(crate) fn new(mut options: ClientOptions) -> Result<Self> {
+    pub(crate) fn new(options: ClientOptions) -> Result<Self> {
         let description = TopologyDescription::new(options.clone())?;
         let is_load_balanced = description.topology_type() == TopologyType::LoadBalanced;
 
@@ -170,8 +97,6 @@ impl Topology {
             handler.handle_topology_opening_event(event);
         }
 
-        let hosts: Vec<_> = options.hosts.drain(..).collect();
-
         let common = Common {
             is_alive: Arc::new(AtomicBool::new(true)),
             message_manager: TopologyMessageManager::new(),
@@ -181,17 +106,6 @@ impl Topology {
 
         let http_client = HttpClient::default();
 
-        #[cfg(test)]
-        let topology_state = TopologyState {
-            description,
-            servers: Default::default(),
-            http_client,
-            mocked: false,
-            id,
-            options: options.clone(),
-        };
-
-        #[cfg(not(test))]
         let topology_state = TopologyState {
             description,
             servers: Default::default(),
@@ -206,13 +120,8 @@ impl Topology {
         // we can block in place here because we're the only ones with access to the lock, so it
         // should be acquired immediately.
         let mut topology_state = RUNTIME.block_in_place(topology.state.write());
-        for address in hosts {
-            topology_state.add_new_server(address, options.clone(), &topology.downgrade());
-        }
-
-        // When the client is in load balanced mode, it doesn't poll for changes in the SRV record.
-        if !is_load_balanced {
-            SrvPollingMonitor::start(topology.downgrade());
+        for address in &options.hosts {
+            topology_state.add_new_server(address.clone(), options.clone(), &topology.downgrade());
         }
 
         if let Some(ref handler) = options.sdam_event_handler {
@@ -232,7 +141,31 @@ impl Topology {
             }
         }
 
-        SrvPollingMonitor::start(topology.downgrade());
+        if is_load_balanced {
+            for server_address in &options.hosts {
+                // Load-balanced clients don't have a heartbeat monitor, so we synthesize
+                // updating the server to `ServerType::LoadBalancer` with an RTT of 0 so it'll
+                // be selected.
+                let new_desc = ServerDescription {
+                    server_type: ServerType::LoadBalancer,
+                    average_round_trip_time: Some(Duration::from_nanos(0)),
+                    ..ServerDescription::new(server_address.clone(), None)
+                };
+                topology_state
+                    .update(new_desc, &options, topology.downgrade())
+                    .map_err(Error::internal)?;
+            }
+        }
+        #[cfg(test)]
+        let disable_monitoring_threads = options
+            .test_options
+            .map(|to| to.disable_monitoring_threads)
+            .unwrap_or(false);
+        #[cfg(not(test))]
+        let disable_monitoring_threads = false;
+        if !is_load_balanced && !disable_monitoring_threads {
+            SrvPollingMonitor::start(topology.downgrade());
+        }
 
         drop(topology_state);
         Ok(topology)
@@ -534,6 +467,11 @@ impl Topology {
             .map(|(addr, server)| (addr.clone(), Arc::downgrade(server)))
             .collect()
     }
+
+    #[cfg(test)]
+    pub(crate) async fn sync_workers(&self) {
+        self.state.read().await.sync_workers().await;
+    }
 }
 
 impl Drop for TopologyState {
@@ -597,7 +535,11 @@ impl TopologyState {
         self.servers.insert(address, server);
 
         #[cfg(test)]
-        if self.mocked {
+        if options
+            .test_options
+            .map(|to| to.disable_monitoring_threads)
+            .unwrap_or(false)
+        {
             return;
         }
 
@@ -702,6 +644,16 @@ impl TopologyState {
 
         self.servers.retain(|host, _| hosts.contains(host));
     }
+
+    #[cfg(test)]
+    async fn sync_workers(&self) {
+        let rxen: FuturesUnordered<_> = self
+            .servers
+            .values()
+            .map(|v| v.pool.sync_worker())
+            .collect();
+        let _: Vec<_> = rxen.collect().await;
+    }
 }
 
 /// Enum describing a point in time during an operation's execution relative to when the MongoDB
